@@ -1,5 +1,5 @@
 # Tweepy
-# Copyright 2009-2022 Joshua Roesslein
+# Copyright 2009-2023 Joshua Roesslein
 # See LICENSE for details.
 
 """
@@ -21,4 +21,4 @@ except ModuleNotFoundError:
 
 from tweepy.asynchronous.client import AsyncClient
 from tweepy.asynchronous.pagination import AsyncPaginator
-from tweepy.asynchronous.streaming import AsyncStream, AsyncStreamingClient
+from tweepy.asynchronous.streaming import AsyncStreamingClient
