@@ -1,5 +1,5 @@
 # Tweepy
-# Copyright 2009-2022 Joshua Roesslein
+# Copyright 2009-2023 Joshua Roesslein
 # See LICENSE for details.
 
 import warnings
@@ -8,6 +8,37 @@ from tweepy.mixins import DataMapping, HashableID
 from tweepy.utils import parse_datetime
 
 
+#: All the potential publically-available fields for :class:`Tweet` objects
+PUBLIC_TWEET_FIELDS = [
+    "attachments",
+    "author_id",
+    "context_annotations",
+    "conversation_id",
+    "created_at",
+    "edit_controls",
+    "edit_history_tweet_ids",
+    "entities",
+    "geo",
+    "id",
+    "in_reply_to_user_id",
+    "lang",
+    "possibly_sensitive",
+    "public_metrics",
+    "referenced_tweets",
+    "reply_settings",
+    "source",
+    "text",
+    "withheld",
+]
+
+#: All the potential fields for :class:`Tweet` objects
+TWEET_FIELDS = PUBLIC_TWEET_FIELDS + [
+    "non_public_metrics",
+    "organic_metrics",
+    "promoted_metrics",
+]
+
+
 class Tweet(HashableID, DataMapping):
     """Tweets are the basic building block of all things Twitter. The Tweet
     object has a long list of ‘root-level’ fields, such as ``id``, ``text``,
@@ -102,6 +133,11 @@ class Tweet(HashableID, DataMapping):
         "everyone", "mentioned_users", and "followers".
     source : str | None
         The name of the app the user Tweeted from.
+
+        .. note::
+
+            As of December 20, 2022, this field has been removed from the Tweet
+            payload. [#]_
     withheld : dict | None
         When present, contains withholding details for `withheld content`_.
 
